defmodule DecentralisedBookIndexWeb.AuthorLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header class="pb-2">
      Listing Authors
    </.header>

    <.search_resources
      search_query={@search_query}
      select_options={@select_options}
      selected_option={@sort_by}
      current_user={@current_user}
      resource_type="author"
      resource_new_url={~p"/authors/new"}
    />

    <%= if Enum.empty?(@page.results) do %>
      <div class="flex justify-center ">
        <div>
          <p class="text-xl font-semibold py-5 dark:text-white">
            No Authors
          </p>
        </div>
      </div>
    <% else %>
      <div class="flex flex-wrap flex-[3_1_auto]">
        <%= for author <- @page.results do %>
          <.author_card
            author={author}
            current_user={@current_user}
          />
        <% end %>
      </div>

      <.pagination
        endpoint={~p"/authors"}
        page={@page}
        page_params={@page_params}
        params={@params}
      />
    <% end %>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name") |> validate_sort_by()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = Metadata.search_author!(
      search_query,
      query: [sort_input: sort_by],
      page: page_params ++ [count: true],
      actor: socket.assigns.current_user
    )

    socket =
      socket
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Author")
    |> assign(
      :author,
      Ash.get!(DecentralisedBookIndex.Metadata.Author, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Author")
    |> assign(:author, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Authors")
    |> assign(:author, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.AuthorLive.FormComponent, {:saved, _author}}, socket) do
    {:noreply, socket}
  end

  defp sort_options do
    [
      {"Sort by name", "name"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"},
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-sort", %{"sort_by" => sort_by}, socket) do
    params =
      socket.assigns.params
      |> Map.put("sort_by", sort_by)

    {:noreply, push_patch(socket, to: ~p"/authors?#{params}")}
  end

  @impl true
  def handle_event("search", %{"query" => query}, socket) do
    params =
      socket.assigns.params
      |> Map.put("query", query)

    {:noreply, push_patch(socket, to: ~p"/authors?#{params}")}
  end
end
