defmodule DecentralisedBookIndexWeb.BookLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header class="pb-2">
      Listing Books
    </.header>

    <.book_search
      search_query={@search_query}
      search_mode={@search_mode}
      search_mode_options={@search_mode_options}
      select_options={@select_options}
      selected_option={@sort_by}
      current_user={@current_user}
    />

    <%= if Enum.empty?(@page.results) do %>
      <div class="flex justify-center ">
        <div>
          <p class="text-xl font-semibold py-5 dark:text-white">
            No Books
          </p>
        </div>
      </div>
    <% else %>
      <div class="flex flex-wrap flex-[3_1_auto]">
        <%= for book <- @page.results do %>
          <.book_card
            book={book}
            current_user={@current_user}
          />
        <% end %>
      </div>

      <.pagination
        endpoint={~p"/books"}
        page={@page}
        page_params={@page_params}
        params={@params}
      />
    <% end %>

    <.modal :if={@live_action in [:new, :edit]} id="book-modal" show on_cancel={JS.patch(~p"/books")}>
      <.live_component
        module={DecentralisedBookIndexWeb.BookLive.FormComponent}
        id={(@book && @book.id) || :new}
        title={@page_title}
        current_user={@current_user}
        action={@live_action}
        book={@book}
        patch={~p"/books"}
      />
    </.modal>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, socket}
  end

  @impl true
  def handle_params(params, _url, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "title") |> validate_sort_by()
    search_mode = Map.get(params, "search_mode", "title") |> validate_search_mode()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = search(
      search_mode,
      search_query,
      page_params,
      sort_by: sort_by,
      actor: socket.assigns[:current_user]
    )

    socket =
      socket
      |> stream(
        :books,
        Ash.read!(DecentralisedBookIndex.Metadata.Book, actor: socket.assigns[:current_user])
      )
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:search_mode, search_mode)
      |> assign(:search_mode_options, search_mode_options())
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> assign_new(:current_user, fn -> nil end)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Book")
    |> assign(
      :book,
      Ash.get!(DecentralisedBookIndex.Metadata.Book, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Book")
    |> assign(:book, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Books")
    |> assign(:book, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.BookLive.FormComponent, {:saved, _book}}, socket) do
    {:noreply, socket}
  end

  defp sort_options do
    [
      {"Sort by title", "title"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"},
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-sort", %{"sort_by" => sort_by}, socket) do
    params =
      socket.assigns.params
      |> Map.put("sort_by", sort_by)

    {:noreply, push_patch(socket, to: ~p"/books?#{params}")}
  end

  defp search_mode_options do
    [
      {"Search by title", "title"},
      {"Search by isbn13", "isbn13"},
      {"Search by isbn", "isbn"},
      {"Search by asin", "asin"},
    ]
  end

  def validate_search_mode(key) do
    valid_keys = Enum.map(search_mode_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-search-mode", %{"search_mode" => search_mode}, socket) do
    params =
      socket.assigns.params
      |> Map.put("search_mode", search_mode)

    {:noreply, push_patch(socket, to: ~p"/books?#{params}")}
  end

  def search(search_mode, search_query, page_params, opts \\ []) do
    sort_by = Keyword.get(opts, :sort_by, nil)
    actor   = Keyword.get(opts, :actor, nil)

    case search_mode do
      "title" ->
        Metadata.search_book!(
          search_query,
          query: [sort_input: sort_by],
          page: page_params ++ [count: true],
          actor: actor
        )

      type ->
        Metadata.search_book_by_bid!(
          type,
          search_query,
          query: [sort_input: sort_by],
          page: page_params ++ [count: true],
          actor: actor
        )
    end
  end

  @impl true
  def handle_event("search", %{"query" => query}, socket) do
    params =
      socket.assigns.params
      |> Map.put("query", query)

    {:noreply, push_patch(socket, to: ~p"/books?#{params}")}
  end

end
