defmodule DecentralisedBookIndexWeb.PublisherLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing Publishers
    </.header>

    <div class="pt-2">
      <.search_resources
        search_query={@search_query}
        select_options={@select_options}
        selected_option={@sort_by}
        current_user={@current_user}
        resource_type="publisher"
        resource_new_url={~p"/publishers/new"}
      />
    </div>

    <%= if Enum.empty?(@page.results) do %>
      <div class="flex justify-center ">
        <div>
          <p class="text-xl font-semibold py-5 dark:text-white">
            No Publisher
          </p>
        </div>
      </div>
    <% else %>
      <div class="pt-2 flex flex-col gap-2">
        <.table
          id="publishers"
          rows={@page.results}
          row_click={fn publisher -> JS.navigate(~p"/publishers/#{publisher}") end}
        >
          <:col :let={publisher} label="Name">{publisher.name}</:col>

          <:action :let={publisher}>
            <div class="sr-only">
              <.link navigate={~p"/publishers/#{publisher}"}>Show</.link>
            </div>

            <.link patch={~p"/publishers/#{publisher}/edit"}>Edit</.link>
          </:action>
        </.table>

        <.pagination
          endpoint={~p"/publishers"}
          page={@page}
          page_params={@page_params}
          params={@params}
        />
      </div>
    <% end %>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name") |> validate_sort_by()
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = Metadata.search_publisher!(
      search_query,
      query: [sort_input: sort_by],
      page: page_params ++ [count: true],
      actor: socket.assigns.current_user
    )

    socket =
      socket
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:select_options, sort_options())
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Publisher")
    |> assign(
      :publisher,
      Ash.get!(DecentralisedBookIndex.Metadata.Publisher, id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Publisher")
    |> assign(:publisher, nil)
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Publishers")
    |> assign(:publisher, nil)
  end

  @impl true
  def handle_info(
        {DecentralisedBookIndexWeb.PublisherLive.FormComponent, {:saved, _publisher}},
        socket
      ) do
    {:noreply, socket}
  end

  defp sort_options do
    [
      {"Sort by name", "name"},
      {"Sort by recently updated", "-updated_at"},
      {"Sort by recently added", "-inserted_at"},
    ]
  end

  def validate_sort_by(key) do
    valid_keys = Enum.map(sort_options(), &elem(&1, 1))

    if key in valid_keys do
      key
    else
      List.first(valid_keys)
    end
  end

  @impl true
  def handle_event("change-sort", %{"sort_by" => sort_by}, socket) do
    params =
      socket.assigns.params
      |> Map.put("sort_by", sort_by)

    {:noreply, push_patch(socket, to: ~p"/publishers?#{params}")}
  end

  @impl true
  def handle_event("search", %{"query" => query}, socket) do
    params =
      socket.assigns.params
      |> Map.put("query", query)

    {:noreply, push_patch(socket, to: ~p"/publishers?#{params}")}
  end
end
