defmodule DecentralisedBookIndexWeb.UserLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="user-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <%= if @form.source.type == :create do %>
        <% end %>
        <%= if @form.source.type == :update do %>
          <.input field={@form[:role]} type="select" label="Role" options={@role_options} />
        <% end %>

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
          <.cancel_button phx-click="cancel" phx-target={@myself}>
            Cancel
          </.cancel_button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign(:role_options, role_options())
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"user" => user_params}, socket) do
    {:noreply, assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, user_params))}
  end

  def handle_event("save", %{"user" => user_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: user_params) do
      {:ok, user} ->
        notify_parent({:saved, user})

        socket =
          socket
          |> put_flash(:info, "User #{socket.assigns.form.source.type}d successfully")
          |> push_navigate(to: patch_url(socket.assigns.action, user.id))

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  def handle_event("cancel", _params, socket) do
    user = socket.assigns.user
    user_id = if user == nil, do: nil, else: user.id

    socket =
      socket
      |> push_navigate(to: patch_url(socket.assigns.action, user_id))

    {:noreply, socket}
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{user: user}} = socket) do
    if user do
      form =
        AshPhoenix.Form.for_update(user, :set_role,
          as: "user",
          actor: socket.assigns.current_user
        )

      assign(socket, form: to_form(form))
    else
      socket =
        socket
        |> put_flash(:info, "User not exists successfully")
        |> push_navigate(to: patch_url(socket.assigns.action, nil))

      {:noreply, socket}
    end
  end

  defp patch_url(action, user_id) do
    case action do
      :edit -> ~p"/users/#{user_id}"
      :new -> ~p"/users"
      _ -> ~p"/users"
    end
  end

  defp role_options() do
    [
      {"Regular user", "user"},
      {"Moderator", "moderator"},
      {"Admin", "admin"}
    ]
  end
end
