defmodule DecentralisedBookIndexWeb.Router do
  use DecentralisedBookIndexWeb, :router

  import Oban.Web.Router
  import Oban.Web.Router
  use AshAuthentication.Phoenix.Router

  import AshAuthentication.Plug.Helpers

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_live_flash
    plug :put_root_layout, html: {DecentralisedBookIndexWeb.Layouts, :root}
    plug :protect_from_forgery
    plug :put_secure_browser_headers
    plug :load_from_session
  end

  pipeline :api do
    plug :accepts, ["json"]
    plug :load_from_bearer
    plug :set_actor, :user
  end

  scope "/api/v1/json" do
    pipe_through [:api]

    forward "/swaggerui",
            OpenApiSpex.Plug.SwaggerUI,
            path: "/api/v1/json/open_api",
            default_model_expand_depth: 4

    forward "/", DecentralisedBookIndexWeb.AshJsonApiRouter
  end

  scope "/", DecentralisedBookIndexWeb do
    pipe_through :browser

    ash_authentication_live_session :moderator_authenticated_routes,
      on_mount: {DecentralisedBookIndexWeb.LiveUserAuth, :moderator_required} do
      live "/books/new", BookLive.Edit, :new
      live "/books/:alternative_book_id/new", BookLive.Edit, :new
      live "/books/:id/edit", BookLive.Edit, :edit

      live "/authors/new", AuthorLive.Edit, :new
      live "/authors/:alternative_author_id/new", AuthorLive.Edit, :new
      live "/authors/:id/edit", AuthorLive.Edit, :edit

      live "/publishers", PublisherLive.Index, :index
      live "/publishers/new", PublisherLive.Edit, :new
      live "/publishers/:id/edit", PublisherLive.Edit, :edit

      live "/publishers/:id", PublisherLive.Show, :show
    end

    ash_authentication_live_session :admin_authenticated_routes,
      on_mount: {DecentralisedBookIndexWeb.LiveUserAuth, :admin_required} do
      live "/servers", DbiServerLive.Index, :index
      live "/servers/new", DbiServerLive.Edit, :new
      live "/servers/:id/edit", DbiServerLive.Edit, :edit

      live "/servers/:id", DbiServerLive.Show, :show

      live "/users", UserLive.Index, :index
      live "/users/:id", UserLive.Show, :show
      live "/users/:id/edit", UserLive.Edit, :edit
    end

    ash_authentication_live_session :maybe_authenticated_routes,
      on_mount: {DecentralisedBookIndexWeb.LiveUserAuth, :live_user_optional} do
      # in each liveview, add one of the following at the top of the module:
      #
      # If an authenticated user must be present:
      # on_mount {DecentralisedBookIndexWeb.LiveUserAuth, :live_user_required}
      #
      # If an authenticated user *may* be present:
      # on_mount {DecentralisedBookIndexWeb.LiveUserAuth, :live_user_optional}
      #
      # If an authenticated user must *not* be present:
      # on_mount {DecentralisedBookIndexWeb.LiveUserAuth, :live_no_user}

      live "/", BookLive.Index, :index

      live "/books", BookLive.Index, :index
      live "/books/:id", BookLive.Show, :show

      live "/authors", AuthorLive.Index, :index
      live "/authors/:id", AuthorLive.Show, :show
    end
  end

  scope "/", DecentralisedBookIndexWeb do
    pipe_through :browser

    # get "/", PageController, :home
    auth_routes AuthController, DecentralisedBookIndex.Accounts.User, path: "/auth"
    sign_out_route AuthController

    # Remove these if you'd like to use your own authentication views
    sign_in_route register_path: "/register",
                  reset_path: "/reset",
                  auth_routes_prefix: "/auth",
                  on_mount: [{DecentralisedBookIndexWeb.LiveUserAuth, :live_no_user}],
                  overrides: [
                    DecentralisedBookIndexWeb.AuthOverrides,
                    AshAuthentication.Phoenix.Overrides.Default
                  ]

    # Remove this if you do not want to use the reset password feature
    reset_route auth_routes_prefix: "/auth",
                overrides: [
                  DecentralisedBookIndexWeb.AuthOverrides,
                  AshAuthentication.Phoenix.Overrides.Default
                ]
  end

  # Other scopes may use custom stacks.
  # scope "/api", DecentralisedBookIndexWeb do
  #   pipe_through :api
  # end

  # Enable LiveDashboard and Swoosh mailbox preview in development
  if Application.compile_env(:decentralised_book_index, :dev_routes) do
    # If you want to use the LiveDashboard in production, you should put
    # it behind authentication and allow only admins to access it.
    # If your application does not have an admins-only section yet,
    # you can use Plug.BasicAuth to set up some basic authentication
    # as long as you are also using SSL (which you should anyway).
    import Phoenix.LiveDashboard.Router

    scope "/dev" do
      pipe_through :browser

      live_dashboard "/dashboard", metrics: DecentralisedBookIndexWeb.Telemetry
      forward "/mailbox", Plug.Swoosh.MailboxPreview
    end

    scope "/" do
      pipe_through :browser

      oban_dashboard("/oban")
    end
  end
end
